
# For m create a matrix representing point on M_0,m
# -------------  Input:
# m              an integer >= 4 ::Int64
# -------------  Output:
# sigma          a matrix ::AbstractAlgebra.Generic.MatSpaceElem{ZZMPolyRingElem}
# R              a ring of sigma ::ZZMPolyRing
# x              vars of R ::Vector{ZZMPolyRingElem}

function create_point_of_M0m(m)
    
    R, x = polynomial_ring(Oscar.QQ, ["x$i" for i = 3:m-1])

    row1 = [ones(Int, m-1); 0]
    row2 = vcat([zero(R), one(R)], x, [one(R)])
    
    sigma = AbstractAlgebra.matrix(R, [transpose(row1); transpose(row2)])

    return sigma, R, x
end

# For a point on M_0,m create an arrangement of maximal minors
# -------------  Input:
# sigma          a matrix ::AbstractAlgebra.Generic.MatSpaceElem{ZZMPolyRingElem}
# R              a ring of sigma ::ZZMPolyRing
# x              vars of R ::Vector{ZZMPolyRingElem}
# -------------  Output:
#                a matrix ::Matrix{Int64}

function create_arrangement(sigma, R, x)
    
    m = size(sigma, 2)
    plucker = minors(sigma, 2)[2:binomial(m,2) - (m-1)]
    all_terms = vcat([one(R)], x)
    
    matrix = zeros(Oscar.QQ,length(all_terms), length(plucker))

    for (i, term) in enumerate(all_terms) 
        for (j, p) in enumerate(plucker)
            matrix[i, j] = AbstractAlgebra.coeff(p, term)
        end
    end
    
    return Int.(matrix)
end

# For a matrix L check that a submatrix L_I associated to columns from I is equivalent to some arrangement of M_{0,k}
# -------------  Input:
# L          a matrix of an arrangement of M_{0,m}
# I          a subset of a ground set of M(L) = {0,...,n}, where n = m(m-3)/2

function check_submatrix(L, I)
    L1 = L[:,I];
    n,d = size(L1);
    
    L3 = L1[setdiff(collect(1:n), findall(j -> all(i -> (L1[j,i] == 0), collect(1:d)), collect(1:n))), :];
    
    sigma2, R2, x2 = create_point_of_M0m(rank(L1) + 2)
    L2 = create_arrangement(sigma2, R2, x2)
    return L3 == L2
end

# For a matrix L of an arrangement of M_{0,m} return all type 2 flats of a given rank
# -------------  Input:
# L          a matrix of an arrangement of M_{0,m}
# r          rank of flats :: Int64
# -------------  Output:
#   a vector of all type 2 flats of L 
function type2_flats(L, r)
    M = matroid_from_matrix_columns(matrix_space(Oscar.QQ,size(L)...)(L))

    return filter(f -> (check_submatrix(L, f)), flats(M, r))
end